﻿// @author Varrattu
// @date 2020-04-30

using maddox.game;
using maddox.game.world;
using maddox.GP;
using System;
using System.Globalization;

public class Mission : AMission
{
	#region === Variablen [voreingestellt für Bf109E-1] ===

	double zielObjektEntfernung = 50;
	double zielObjektHorizontalVerschieben = 0.00;
	double zielObjektVertikalVerschieben = 0.62;

	#endregion Variablen

	public override void Inited()
	{
		base.Inited();
		MissionNumberListener = -1;
		SetMainMenu(GamePlay.gpPlayer());
		
	}

	public override void OnPlaceEnter(Player player, AiActor actor, int placeIndex)
	{
		base.OnPlaceEnter(player, actor, placeIndex);

		AiAircraft airplane = GamePlay.gpPlayer().Place() as AiAircraft;
		if (airplane == null) return;

		Timeout(0.2, () =>
		{
			SetTarget(airplane, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);

		});
	}

	private void SetTarget(AiAircraft aircraft, double zielObjektEntfernung, double zielObjektVertikalVerschieben, double zielObjektHorizontalVerschieben)
	{
		double nickAchsenHöheAGL = aircraft.getParameter(part.ParameterTypes.Z_AltitudeAGL, -1);
		if (nickAchsenHöheAGL > 10) return;

		ISectionFile secFile = GamePlay.gpCreateSectionFile();

		double rwskDEG = Convert.ToInt32(aircraft.getParameter(part.ParameterTypes.Z_Orientation, 0));
		double rwskRAD = rwskDEG * Math.PI / 180;

		double zielObjektKorrigiertCOS = zielObjektHorizontalVerschieben * Math.Cos(rwskRAD);

		double zielObjektEbenePx = zielObjektEntfernung * Math.Cos(rwskRAD) + aircraft.Pos().x - zielObjektKorrigiertCOS;
		double zielObjektBasisPy = zielObjektEntfernung * Math.Sin(rwskRAD) + aircraft.Pos().y - zielObjektKorrigiertCOS;

		double zielRwSKdeg = rwskDEG - 180;

		double nickWinkelDEG = aircraft.getParameter(part.ParameterTypes.Z_Orientation, 1) * -1;
		
		double zielObjektBasisRAW = zielObjektEntfernung * Math.Sin(nickWinkelDEG * Math.PI / 180);
		
		double visierSchuss = zielObjektBasisRAW  + nickAchsenHöheAGL + zielObjektVertikalVerschieben;

		double zielObjektBasisPz = visierSchuss - 1.90;
		// <hstart> bestimmt die Höhe des iL2CoD-Objekts über Grund [AGL)
		// <Airfield.Ground_target_1> um -1.90 [m] AGL korrigiert

		double hstart = zielObjektBasisPz;

		GamePlay.gpLogServer(null, "\n", null);
		GamePlay.gpLogServer(null, "************ ~oOo~ ************", null);
		GamePlay.gpLogServer(null, "         Nickwinkel [Grad]: " + nickWinkelDEG.ToString("0.00°"), null);
		GamePlay.gpLogServer(null, "         Querachse AGL [m]: " + nickAchsenHöheAGL.ToString("0.00m"), null);
		GamePlay.gpLogServer(null, "        Zielentfernung [m]: " + zielObjektEntfernung.ToString("0.00m"), null);
		GamePlay.gpLogServer(null, "  Vertikal korrigiert [cm]: " + (zielObjektVertikalVerschieben * 100).ToString("0cm"), null);
		GamePlay.gpLogServer(null, "Horizontal korrigiert [cm]: " + (zielObjektHorizontalVerschieben * 100).ToString("0cm"), null);
		GamePlay.gpLogServer(null, "************ ~oOo~ ************", null);
		GamePlay.gpLogServer(null, "\n", null);

		string coords =
				" " +
				zielObjektEbenePx.ToString("F2", CultureInfo.CreateSpecificCulture("en-GB")) +
				" " +
				zielObjektBasisPy.ToString("F2", CultureInfo.CreateSpecificCulture("en-GB")) +
				" " +
				zielRwSKdeg.ToString("F2", CultureInfo.CreateSpecificCulture("en-GB")) +
				" /hstart " +
				hstart.ToString("F2", CultureInfo.CreateSpecificCulture("en-GB"))
				;

		string section = "Stationary";
		string key = "AnSchiessen Stationary.Airfield.Ground_target_1 gb";

		secFile.add(section, key, coords);

		GamePlay.gpPostMissionLoad(secFile);
	}

	private void DeleteTarget()
	{
		GroundStationary[] GroundStationaries = GamePlay.gpGroundStationarys();
		foreach (GroundStationary stationary in GroundStationaries)
		{
			if (stationary.Name.Contains("AnSchiessen")) stationary.Destroy();
		}
	}

	public bool aircraftMoved(AiAircraft aircraft)
	{
		return (aircraft.getParameter(part.ParameterTypes.Z_VelocityTAS, -1) > 0.1); // >0.1 m/s == false
	}

	#region === Single Player Menu ===

	enum MenuID
	{
		MainMenu,         // MenuID 0 "Zielabstand ändern", "Ziellage ändern"
		DistanceMenu,     // MenuID 1 Zielabstand +/-
		TargetAdjustment  // MenuID 2 Ziellage +/- 
	}

	public void SetMainMenu(Player player)
	{
		GamePlay.gpSetOrderMissionMenu(player,
			false, (int)MenuID.MainMenu,
			new string[] { "Zielabstand ändern", "Ziellage ändern" },
			new bool[] { true, true });
	}

	public void SetDistanceMenu(Player player)
	{
		GamePlay.gpSetOrderMissionMenu(player,
			true, (int)MenuID.DistanceMenu,
			new string[] { "+50m", "-50m", "", "+5m", "-5m", "", "+1m", "-1m" },
			new bool[] { true, true, true, true, true, true, true, true });
	}

	public void SetTargetAdjustment(Player player)
	{
		GamePlay.gpSetOrderMissionMenu(player,
			true, (int)MenuID.TargetAdjustment,
			new string[] { "höher: +10cm", "tiefer: -10cm", "Höhe +5cm", "Höhe -5cm", "Höhe +1cm", "Höhe -1cm","", "nach links: 1cm", "nach rechts: 1cm "
				},
			new bool[] { true, true, true, true, true, true, true, true, true });
	}

	public override void OnOrderMissionMenuSelected(Player player, int ID, int menuItemIndex)
	{
		base.OnOrderMissionMenuSelected(player, ID, menuItemIndex);

		switch (ID)
		{
			case 0:
				switch (menuItemIndex)
				{
					case 1:
						SetDistanceMenu(player);
						break;
					case 2:
						SetTargetAdjustment(player);
						break;
					case 0:
						SetMainMenu(player);
						break;

				}
				break;

			case 1:
				switch (menuItemIndex)
				{
					case 1:
						DeleteTarget();
						zielObjektEntfernung = zielObjektEntfernung +50;
						if (zielObjektEntfernung > 650) zielObjektEntfernung = 650;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetDistanceMenu(player);
						break;

					case 2:
						DeleteTarget();
						zielObjektEntfernung = zielObjektEntfernung -50;
						if (zielObjektEntfernung < 5) zielObjektEntfernung = 5;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetDistanceMenu(player);
						break;

					case 4:
						DeleteTarget();
						zielObjektEntfernung = zielObjektEntfernung +5;
						if (zielObjektEntfernung > 650) zielObjektEntfernung = 650;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetDistanceMenu(player);
						break;

					case 5:
						DeleteTarget();
						zielObjektEntfernung = zielObjektEntfernung -5;
						if (zielObjektEntfernung < 5) zielObjektEntfernung = 5;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetDistanceMenu(player);
						break;

					case 7:
						DeleteTarget();
						zielObjektEntfernung = zielObjektEntfernung +1;
						if (zielObjektEntfernung > 650) zielObjektEntfernung = 650;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetDistanceMenu(player);
						break;

					case 8:
						DeleteTarget();
						zielObjektEntfernung = zielObjektEntfernung -1;
						if (zielObjektEntfernung < 5) zielObjektEntfernung = 5;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetDistanceMenu(player);
						break;

					case 0:
						SetMainMenu(player);
						break;

					default:
						SetMainMenu(player);
						break;

				}
				break;


			case 2:
				switch (menuItemIndex)
				{
					case 1:
						DeleteTarget();
						zielObjektVertikalVerschieben = zielObjektVertikalVerschieben + 0.1;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetMainMenu(player);
						SetTargetAdjustment(player);
						break;

					case 2:
						DeleteTarget();
						zielObjektVertikalVerschieben = zielObjektVertikalVerschieben - 0.1;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetMainMenu(player);
						SetTargetAdjustment(player);
						break;

					case 3:
						DeleteTarget();
						zielObjektVertikalVerschieben = zielObjektVertikalVerschieben + 0.05;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetMainMenu(player);
						SetTargetAdjustment(player);
						break;

					case 4:
						DeleteTarget();
						zielObjektVertikalVerschieben = zielObjektVertikalVerschieben - 0.05;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetMainMenu(player);
						SetTargetAdjustment(player);
						break;

					case 5:
						DeleteTarget();
						zielObjektVertikalVerschieben = zielObjektVertikalVerschieben + 0.01;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetMainMenu(player);
						SetTargetAdjustment(player);
						break;

					case 6:
						DeleteTarget();
						zielObjektVertikalVerschieben = zielObjektVertikalVerschieben - 0.01;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetMainMenu(player);
						SetTargetAdjustment(player);
						break;

					case 8:
						DeleteTarget();
						zielObjektHorizontalVerschieben = zielObjektHorizontalVerschieben - 0.01;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetMainMenu(player);
						SetTargetAdjustment(player);
						break;

					case 9:
						DeleteTarget();
						zielObjektHorizontalVerschieben = zielObjektHorizontalVerschieben + 0.01;
						SetTarget(GamePlay.gpPlayer().Place() as AiAircraft, zielObjektEntfernung, zielObjektVertikalVerschieben, zielObjektHorizontalVerschieben);
						SetMainMenu(player);
						SetTargetAdjustment(player);
						break;

					case 0:
						SetMainMenu(player);
						break;

				}
				break;
		}
	}
	#endregion Menü

}
